use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::assignment::PlanEventPersonalTablePartialTemplate,
    models::{Assignment, Event, Role, User},
    utils::{
        event_planning_template::{
            generate_availabillity_assignment_list, generate_status_whether_staff_is_required,
        },
        ApplicationError, TemplateResponse,
    },
};

#[derive(Deserialize)]
struct AssignmentDeleteQuery {
    availabillity: i32,
    event: i32,
}

#[actix_web::delete("/assignments/delete")]
pub async fn delete(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<AssignmentDeleteQuery>,
) -> Result<impl Responder, ApplicationError> {
    let Some(event) = Event::read_by_id_including_location(pool.get_ref(), query.event).await?
    else {
        return Ok(HttpResponse::NotFound().finish());
    };

    let user_is_admin_or_area_manager_of_event_area = user.role == Role::Admin
        || (user.role == Role::AreaManager
            && user.area_id == event.location.as_ref().unwrap().area_id);

    if !user_is_admin_or_area_manager_of_event_area {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(assignment) = Assignment::read(pool.get_ref(), event.id, query.availabillity).await?
    else {
        return Ok(HttpResponse::NotFound().finish());
    };

    Assignment::delete(pool.get_ref(), event.id, assignment.availabillity_id).await?;

    let availabillities = generate_availabillity_assignment_list(pool.get_ref(), &event).await?;
    let (
        further_posten_required,
        further_fuehrungsassistent_required,
        further_wachhabender_required,
    ) = generate_status_whether_staff_is_required(pool.get_ref(), &event).await?;

    let template = PlanEventPersonalTablePartialTemplate {
        event,
        availabillities,
        further_posten_required,
        further_fuehrungsassistent_required,
        further_wachhabender_required,
    };

    Ok(template.to_response()?)
}

#[cfg(test)]
mod tests {
    use actix_http::StatusCode;
    use brass_macros::db_test;
    use chrono::{NaiveDate, NaiveTime};
    use fake::{faker::chrono::en::Date, Fake, Faker};
    use sqlx::PgPool;

    use crate::{
        models::{
            Assignment, AssignmentChangeset, Availability, AvailabilityChangeset, Event,
            EventChangeset, Function, Location, Role, User,
        },
        utils::test_helper::{test_delete, DbTestContext, RequestConfig},
    };

    async fn arrange(pool: &PgPool) -> anyhow::Result<()> {
        Location::create(pool, "Location", 1).await?;

        User::create(pool, Faker.fake()).await?;

        let date: NaiveDate = Date().fake();
        let start = NaiveTime::from_hms_opt(10, 0, 0).unwrap();
        let end = NaiveTime::from_hms_opt(15, 30, 0).unwrap();

        let new_event = EventChangeset::create_for_test(date.and_time(start), date.and_time(end));
        Event::create(pool, new_event).await?;

        let new_availability = AvailabilityChangeset {
            time: (
                date.and_hms_opt(12, 0, 0).unwrap(),
                date.and_hms_opt(15, 0, 0).unwrap(),
            ),
            comment: None,
        };
        Availability::create(pool, 1, new_availability).await?;

        let new_assignment = AssignmentChangeset {
            function: Function::Posten,
            time: (date.and_time(start), date.and_time(end)),
        };
        Assignment::create(pool, 1, 1, new_assignment).await?;

        // TODO: check this test after large refactoring
        Ok(())
    }

    #[db_test]
    async fn works_if_assignment_exists_and_actor_is_area_manager(context: &DbTestContext) {
        arrange(&context.db_pool).await.unwrap();

        let app = context.app().await;
        let config = RequestConfig {
            uri: "/assignments/delete?availabillity=1&event=1".to_string(),
            role: Role::AreaManager,
            function: vec![Function::Posten],
            user_area: 1,
        };

        let response = test_delete(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::OK, response.status());
        assert_eq!(
            0,
            Assignment::read_all_by_event(&context.db_pool, 1)
                .await
                .unwrap()
                .len()
        )
    }

    #[db_test]
    async fn doesnt_work_when_actor_is_staff(context: &DbTestContext) {
        arrange(&context.db_pool).await.unwrap();

        let app = context.app().await;
        let config = RequestConfig {
            uri: "/assignments/delete?availabillity=1&event=1".to_string(),
            role: Role::Staff,
            function: vec![Function::Posten],
            user_area: 1,
        };

        let response = test_delete(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::UNAUTHORIZED, response.status());
    }

    #[db_test]
    async fn doesnt_work_when_event_does_not_exist(context: &DbTestContext) {
        let app = context.app().await;
        let config = RequestConfig {
            uri: "/assignments/delete?availabillity=1&event=1".to_string(),
            role: Role::AreaManager,
            function: vec![Function::Posten],
            user_area: 1,
        };

        let response = test_delete(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::NOT_FOUND, response.status());
    }

    #[db_test]
    async fn doesnt_work_when_assignment_does_not_exist(context: &DbTestContext) {
        arrange(&context.db_pool).await.unwrap();

        let app = context.app().await;
        let config = RequestConfig {
            uri: "/assignments/delete?availabillity=2&event=1".to_string(),
            role: Role::AreaManager,
            function: vec![Function::Posten],
            user_area: 1,
        };

        let response = test_delete(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::NOT_FOUND, response.status());
    }
}
