use actix_web::{
    http::header::{ContentDisposition, ContentType, CONTENT_DISPOSITION},
    web, HttpResponse, Responder,
};
use chrono::{Months, NaiveDate, NaiveDateTime, Utc};
use quick_xml::se::Serializer;
use serde::{Deserialize, Serialize};
use sqlx::PgPool;

use crate::{
    models::{Area, Availability, Role, User, UserFunction},
    utils::ApplicationError,
};

#[derive(Deserialize)]
struct ExportQuery {
    year: u16,
    month: u8,
    area_id: Option<i32>,
    format: String,
}

#[derive(Serialize)]
struct Export {
    year: u16,
    month: u8,
    area: String,
    availabillities: Vec<ExportAvailabillity>,
}

#[derive(Serialize)]
struct ExportAvailabillity {
    name: String,
    area: String,
    function: UserFunction,
    start: NaiveDateTime,
    end: NaiveDateTime,
    assigned: bool,
    comment: String,
}

#[actix_web::get("/export/availabilitydata")]
pub async fn get(
    pool: web::Data<PgPool>,
    user: web::ReqData<User>,
    query: web::Query<ExportQuery>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let start_date = NaiveDate::from_ymd_opt(query.year as i32, query.month as u32, 1)
        .unwrap_or(Utc::now().date_naive());
    let end_date = start_date
        .checked_add_months(Months::new(1))
        .unwrap()
        .pred_opt()
        .unwrap();

    let area_id = if user.role == Role::Admin && query.area_id.is_some() {
        query.area_id.unwrap()
    } else {
        user.area_id
    };

    let availabillities =
        Availability::read_for_export(pool.get_ref(), (start_date, end_date), area_id).await?;

    let export_availabillities = availabillities
        .into_iter()
        .map(|a| ExportAvailabillity {
            name: a.user.as_ref().unwrap().name.clone(),
            area: a.user.as_ref().unwrap().area.as_ref().unwrap().name.clone(),
            function: a.user.unwrap().function,
            start: a.start,
            end: a.end,
            assigned: false,
            comment: a.comment.unwrap_or(String::new()),
        })
        .collect();

    let area = Area::read_by_id(pool.get_ref(), area_id)
        .await?
        .unwrap()
        .name;

    let export = Export {
        year: query.year,
        month: query.month,
        area,
        availabillities: export_availabillities,
    };

    let (out, content_type) = match query.format.as_str() {
        "xml" => {
            let mut buffer = String::new();
            let mut ser = Serializer::new(&mut buffer);
            ser.indent(' ', 2);

            export.serialize(ser).unwrap();

            (buffer, ContentType::xml())
        }
        "json" => (
            serde_json::to_string_pretty(&export).unwrap_or_default(),
            ContentType::json(),
        ),
        _ => return Ok(HttpResponse::BadRequest().finish()),
    };

    if !out.is_empty() {
        return Ok(HttpResponse::Ok()
            .content_type(content_type)
            .insert_header((
                CONTENT_DISPOSITION,
                ContentDisposition::attachment(format!("export.{}", query.format)),
            ))
            .body(out));
    }

    Ok(HttpResponse::BadRequest().finish())
}
