use chrono::{NaiveDate, NaiveDateTime};
use sqlx::{query, PgPool};

use super::{Area, AvailabilityChangeset, Result, Role, User, UserFunction};

#[derive(Clone, Debug)]
pub struct Availability {
    pub id: i32,
    pub user_id: i32,
    pub user: Option<User>,
    pub start: NaiveDateTime,
    pub end: NaiveDateTime,
    pub comment: Option<String>,
}

impl Availability {
    // TODO: fix db name
    pub async fn create(
        pool: &PgPool,
        user_id: i32,
        changeset: AvailabilityChangeset,
    ) -> Result<()> {
        query!(
            r#"
            INSERT INTO availabillity (userId, startTimestamp, endTimestamp, comment)
            VALUES ($1, $2, $3, $4);
            "#,
            user_id,
            changeset.time.0.and_utc(),
            changeset.time.1.and_utc(),
            changeset.comment
        )
        .execute(pool)
        .await?;

        Ok(())
    }

    pub async fn read_by_date_and_area_including_user(
        pool: &PgPool,
        date: NaiveDate,
        area_id: i32,
    ) -> Result<Vec<Availability>> {
        let records = query!(
            r##"
                             SELECT
                                 availabillity.id,
                                 availabillity.userId,
                                 availabillity.startTimestamp,
                                 availabillity.endTimestamp,
                                 availabillity.comment,
                                 user_.name,
                                 user_.email,
                                 user_.password,
                                 user_.salt,
                                 user_.role AS "role: Role",
                                 user_.function AS "function: UserFunction",
                                 user_.areaId,
                                 user_.locked,
                                 user_.lastLogin,
                                 user_.receiveNotifications
                             FROM availabillity
                             JOIN user_ ON availabillity.userId = user_.id
                             WHERE availabillity.startTimestamp::date = $1
                             AND user_.areaId = $2;
        "##,
            date,
            area_id
        )
        .fetch_all(pool)
        .await?;
        // TODO: maybe starttimestamp::date = $1 doesnt work?

        let availabillities = records
            .iter()
            .map(|r| Availability {
                id: r.id,
                user_id: r.userid,
                user: Some(User {
                    id: r.userid,
                    name: r.name.clone(),
                    email: r.email.clone(),
                    password: r.password.clone(),
                    salt: r.salt.clone(),
                    role: r.role,
                    function: r.function.clone(),
                    area_id: r.areaid,
                    area: None,
                    locked: r.locked,
                    last_login: r.lastlogin,
                    receive_notifications: r.receivenotifications,
                }),
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
                comment: r.comment.clone(),
            })
            .collect();

        Ok(availabillities)
    }

    pub async fn read_by_date_time_area_including_user(
        pool: &PgPool,
        date: NaiveDate,
        time: (NaiveDateTime, NaiveDateTime),
        area_id: i32,
    ) -> Result<Vec<Availability>> {
        let records = query!(
            r##"
                 SELECT
                     availabillity.id,
                     availabillity.userId,
                     availabillity.startTimestamp,
                     availabillity.endTimestamp,
                     availabillity.comment,
                     user_.name,
                     user_.email,
                     user_.password,
                     user_.salt,
                     user_.role AS "role: Role",
                     user_.function AS "function: UserFunction",
                     user_.areaId,
                     user_.locked,
                     user_.lastLogin,
                     user_.receiveNotifications
                 FROM availabillity
                 JOIN user_ ON availabillity.userId = user_.id
                 WHERE availabillity.starttimestamp::date = $1
                 AND user_.areaId = $2
                 AND availabillity.startTimestamp <= $3 AND availabillity.endTimestamp >= $4;
        "##,
            date,
            area_id,
            time.0.and_utc(),
            time.1.and_utc()
        )
        .fetch_all(pool)
        .await?;

        // TODO: maybe starttimestamp::date = $1 doesnt work?
        let availabillities = records
            .iter()
            .map(|r| Availability {
                id: r.id,
                user_id: r.userid,
                user: Some(User {
                    id: r.userid,
                    name: r.name.clone(),
                    email: r.email.clone(),
                    password: r.password.clone(),
                    salt: r.salt.clone(),
                    role: r.role,
                    function: r.function.clone(),
                    area_id: r.areaid,
                    area: None,
                    locked: r.locked,
                    last_login: r.lastlogin,
                    receive_notifications: r.receivenotifications,
                }),
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
                comment: r.comment.clone(),
            })
            .collect();

        Ok(availabillities)
    }

    pub async fn read_by_id_including_user(pool: &PgPool, id: i32) -> Result<Option<Availability>> {
        let record = query!(
            r##"
                             SELECT
                                 availabillity.id,
                                 availabillity.userId,
                                 availabillity.startTimestamp,
                                 availabillity.endTimestamp,
                                 availabillity.comment,
                                 user_.name,
                                 user_.email,
                                 user_.password,
                                 user_.salt,
                                 user_.role AS "role: Role",
                                 user_.function AS "function: UserFunction",
                                 user_.areaId,
                                 user_.locked,
                                 user_.lastLogin,
                                 user_.receiveNotifications
                             FROM availabillity
                             LEFT JOIN assignment ON availabillity.Id = assignment.availabillityId
                             JOIN user_ ON availabillity.userId = user_.id
                             WHERE availabillity.id = $1;
        "##,
            id
        )
        .fetch_optional(pool)
        .await?;

        let availabillity = record.map(|r| Availability {
            id: r.id,
            user_id: r.userid,
            user: Some(User {
                id: r.userid,
                name: r.name.clone(),
                email: r.email.clone(),
                password: r.password.clone(),
                salt: r.salt.clone(),
                role: r.role,
                function: r.function.clone(),
                area_id: r.areaid,
                area: None,
                locked: r.locked,
                last_login: r.lastlogin,
                receive_notifications: r.receivenotifications,
            }),
            start: r.starttimestamp.naive_utc(),
            end: r.endtimestamp.naive_utc(),
            comment: r.comment.clone(),
        });

        Ok(availabillity)
    }

    pub async fn read_by_id(pool: &PgPool, id: i32) -> Result<Option<Availability>> {
        let record = query!("SELECT * FROM availabillity WHERE id = $1", id)
            .fetch_optional(pool)
            .await?;

        let availabillity = record.map(|record| Availability {
            id: record.id,
            user_id: record.userid,
            user: None,
            start: record.starttimestamp.naive_utc(),
            end: record.endtimestamp.naive_utc(),
            comment: record.comment.clone(),
        });

        Ok(availabillity)
    }

    pub async fn read_for_export(
        pool: &PgPool,
        date_range: (NaiveDate, NaiveDate),
        area_id: i32,
    ) -> Result<Vec<Availability>> {
        let records = query!(
            r##"
                             SELECT
                                 availabillity.id,
                                 availabillity.userId,
                                 availabillity.startTimestamp,
                                 availabillity.endTimestamp,
                                 availabillity.comment,
                                 user_.name,
                                 user_.email,
                                 user_.password,
                                 user_.salt,
                                 user_.role AS "role: Role",
                                 user_.function AS "function: UserFunction",
                                 user_.areaId,
                                 user_.locked,
                                 user_.lastLogin,
                                 user_.receiveNotifications,
                                 area.name AS areaName
                             FROM availabillity
                             JOIN user_ ON availabillity.userId = user_.id
                             JOIN area ON user_.areaId = area.id
                             WHERE user_.areaId = $1 AND
                             availabillity.starttimestamp::date >= $2 AND
                             availabillity.starttimestamp::date <= $3;
        "##,
            area_id,
            date_range.0,
            date_range.1
        )
        .fetch_all(pool)
        .await?;
        // TODO: maybe starttimestamp::date = $1 doesnt work?

        let availabillities = records
            .iter()
            .map(|r| Availability {
                id: r.id,
                user_id: r.userid,
                user: Some(User {
                    id: r.userid,
                    name: r.name.clone(),
                    email: r.email.clone(),
                    password: r.password.clone(),
                    salt: r.salt.clone(),
                    role: r.role,
                    function: r.function.clone(),
                    area_id: r.areaid,
                    area: Some(Area {
                        id: r.areaid,
                        name: r.areaname.clone(),
                    }),
                    locked: r.locked,
                    last_login: r.lastlogin,
                    receive_notifications: r.receivenotifications,
                }),
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
                comment: r.comment.clone(),
            })
            .collect();

        Ok(availabillities)
    }

    pub async fn read_by_user_and_date(
        pool: &PgPool,
        user_id: i32,
        date: &NaiveDate,
    ) -> Result<Vec<Availability>> {
        let records = query!(
            r##"
                             SELECT
                                 availabillity.id,
                                 availabillity.userId,
                                 availabillity.startTimestamp,
                                 availabillity.endTimestamp,
                                 availabillity.comment
                             FROM availabillity
                             WHERE availabillity.userId = $1
                             AND availabillity.starttimestamp::date = $2;
        "##,
            user_id,
            date
        )
        .fetch_all(pool)
        .await?;
        // TODO: maybe starttimestamp::date = $1 doesnt work?

        let availabillities = records
            .iter()
            .map(|r| Availability {
                id: r.id,
                user_id: r.userid,
                user: None,
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
                comment: r.comment.clone(),
            })
            .collect();

        Ok(availabillities)
    }

    pub async fn update(pool: &PgPool, id: i32, changeset: AvailabilityChangeset) -> Result<()> {
        query!(
            "UPDATE availabillity SET startTimestamp = $1, endTimestamp = $2, comment = $3 WHERE id = $4",
            changeset.time.0.and_utc(),
            changeset.time.1.and_utc(),
            changeset.comment,
            id
        )
        .execute(pool)
        .await?;

        Ok(())
    }

    pub async fn delete(pool: &PgPool, id: i32) -> Result<()> {
        query!("DELETE FROM availabillity WHERE id = $1", id)
            .execute(pool)
            .await?;
        Ok(())
    }
}
