use chrono::{NaiveDate, NaiveDateTime};
use sqlx::{query, PgPool};

use super::Result;

pub struct VehicleAssignement {
    pub event_id: i32,
    pub vehicle_id: i32,
    pub start: NaiveDateTime,
    pub end: NaiveDateTime,
}

impl VehicleAssignement {
    pub async fn create(
        pool: &PgPool,
        event_id: i32,
        vehicle_id: i32,
        start: NaiveDateTime,
        end: NaiveDateTime,
    ) -> Result<()> {
        query!(
            "INSERT INTO vehicleassignement (eventId, vehicleId, startTimestamp, endTimestamp) VALUES ($1, $2, $3, $4);",
            event_id,
            vehicle_id,
            start.and_utc(),
            end.and_utc()
        )
        .execute(pool)
        .await?;

        Ok(())
    }

    pub async fn read(
        pool: &PgPool,
        event_id: i32,
        vehicle_id: i32,
    ) -> Result<Option<VehicleAssignement>> {
        let record = query!("SELECT * FROM vehicleAssignement WHERE vehicleAssignement.eventId = $1 AND vehicleAssignement.vehicleId = $2;", event_id, vehicle_id).fetch_optional(pool)
            .await?;

        let vehicle_assignment = record.map(|r| VehicleAssignement {
            event_id: r.eventid,
            vehicle_id: r.vehicleid,
            start: r.starttimestamp.naive_utc(),
            end: r.endtimestamp.naive_utc(),
        });

        Ok(vehicle_assignment)
    }

    pub async fn read_all_by_event(
        pool: &PgPool,
        event_id: i32,
    ) -> Result<Vec<VehicleAssignement>> {
        let records = query!(
            "SELECT * FROM vehicleAssignement WHERE vehicleAssignement.eventId = $1;",
            event_id
        )
        .fetch_all(pool)
        .await?;

        let vehicle_assignments = records
            .iter()
            .map(|r| VehicleAssignement {
                event_id: r.eventid,
                vehicle_id: r.vehicleid,
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
            })
            .collect();

        Ok(vehicle_assignments)
    }

    pub async fn read_all_by_vehicle_and_date(
        pool: &PgPool,
        vehicle_id: i32,
        date: NaiveDate,
    ) -> Result<Vec<VehicleAssignement>> {
        let records = query!(
            r#"
            SELECT
                vehicleAssignement.eventId,
                vehicleAssignement.vehicleId,
                vehicleAssignement.starttimestamp,
                vehicleAssignement.endtimestamp
            FROM vehicleAssignement
            JOIN event ON vehicleAssignement.eventId = event.id
            WHERE vehicleAssignement.vehicleid = $1
            AND event.starttimestamp::date = $2;
            "#,
            vehicle_id,
            date,
        )
        .fetch_all(pool)
        .await?;

        // TODO: maybe ::date doesnt work

        let vehicle_assignments = records
            .iter()
            .map(|r| VehicleAssignement {
                event_id: r.eventid,
                vehicle_id: r.vehicleid,
                start: r.starttimestamp.naive_utc(),
                end: r.endtimestamp.naive_utc(),
            })
            .collect();

        Ok(vehicle_assignments)
    }

    pub async fn delete(pool: &PgPool, event_id: i32, vehicle_id: i32) -> Result<()> {
        query!(
            "DELETE FROM vehicleassignement WHERE eventId = $1 AND vehicleId = $2;",
            event_id,
            vehicle_id
        )
        .execute(pool)
        .await?;

        Ok(())
    }
}
